#!perl

use strict;
use warnings;

use Inline qw/C/;

my $fieldstring = $ARGV[0];
if ( ! $fieldstring or $fieldstring !~ /^[\d.]{81}$/ ) {
    die 'Fehlerhaftes Feld!';
}

my @block = (
    [ 1, 2 ], [ 0, 2 ], [ 0, 1 ],
    [ 4, 5 ], [ 3, 5 ], [ 3, 4 ],
    [ 7, 8 ], [ 6, 8 ], [ 6, 7 ],
);
my @fieldr = map{ [ split //, $_ ] } $fieldstring =~ /(.{9})/g;
my @empty = 
    map{ @$_ } 
    sort{ @$a <=> @$b } 
    map{ 
        my $y = $_;
        [ map{ [ $_, $y ] }grep{ $fieldr[$y]->[$_] eq '.' } 0..8 ]
    } 0..8;
my %candidates;

for my $xy ( reverse @empty ) {
    push_empty( @$xy );
}
populate_field( \@fieldr );
print_field();

brute_force( $#empty );

__END__
__C__
int block[9][2] = {
    { 1, 2 }, { 0, 2 }, { 0, 1 },
    { 4, 5 }, { 3, 5 }, { 3, 4 },
    { 7, 8 }, { 6, 8 }, { 6, 7 },
};
int empty[81][2];
int counter = -1;
char field[9][9];

void push_empty( int x, int y ) {
    counter++;
    empty[counter][0] = x;
    empty[counter][1] = y;
}

void populate_field ( AV* fieldAV ) {
    int x, y;
    for ( y = 0; y < 9; y++ ) {
        SV** rowPtr = av_fetch(fieldAV,y,0);
        AV* rowAV = (AV*) SvRV( *rowPtr );
        for ( x = 0; x < 9; x++ ) {
            SV**  numPtr = av_fetch(rowAV,x,0);
            char* num = SvPV_nolen( *numPtr );
            field[y][x] = num[0];
        }
    }
}

void print_field () {
    int x, y;
    for ( y = 0; y < 9; y++ ) {
        for ( x = 0; x < 9; x++ ) {
            printf( "%c ", field[y][x] );
        }
        printf( "\n" );
    }
    printf( "\n" );
}

void brute_force( int emptyc ) {
    int i;
    if ( emptyc == -1 ) {
        print_field();
        return;
    }
    int x = empty[emptyc][0];
    int y = empty[emptyc][1];
    int candidates[10] = { 1,1,1,1,1,1,1,1,1,1 };
    for ( i = 0; i < 9; i++ ) {
        if ( field[y][i] != '.' )
            candidates[ field[y][i] - 48 ] = 0;
    }
    for ( i = 0; i < 9; i++ ) {
        if ( field[i][x] != '.' )
            candidates[ field[i][x] - 48 ] = 0;
    }
    if ( field[block[y][0]][block[x][0]] != '.' )
        candidates[ field[block[y][0]][block[x][0]] - 48 ] = 0;
    if ( field[block[y][0]][block[x][1]] != '.' )
        candidates[ field[block[y][0]][block[x][1]] - 48 ] = 0;
    if ( field[block[y][1]][block[x][0]] != '.' )
        candidates[ field[block[y][1]][block[x][0]] - 48 ] = 0;
    if ( field[block[y][1]][block[x][1]] != '.' )
        candidates[ field[block[y][1]][block[x][1]] - 48 ] = 0;
    for ( i = 1; i <= 9; i++ ) {
        if ( candidates[i] == 1 ) {
            char s[10];
            sprintf( s, "%i", i );
            field[y][x] = s[0];
            brute_force( emptyc - 1 );
        }
    }
    field[y][x] = '.';
}

